-- only show virtual text for WARN and higher
vim.diagnostic.config({
  virtual_text = { severity = { min = vim.diagnostic.severity.WARN } }
})

local nvim_lsp = require('lspconfig')
local null_ls = require('null-ls')

local null_ls_sources = {
  null_ls.builtins.code_actions.gitsigns,
}

-- enable LS / null-ls sources based on executable presence
if vim.fn.executable("node_modules/.bin/eslint") == 1 then
  table.insert(null_ls_sources, null_ls.builtins.formatting.eslint_d)
  table.insert(null_ls_sources, null_ls.builtins.diagnostics.eslint_d)
  table.insert(null_ls_sources, null_ls.builtins.code_actions.eslint_d)
end

if vim.fn.executable("gofmt") == 1 then
  table.insert(null_ls_sources, null_ls.builtins.formatting.gofmt)
end

if vim.fn.executable("shellcheck") == 1 then
  table.insert(null_ls_sources, null_ls.builtins.diagnostics.shellcheck)
  table.insert(null_ls_sources, null_ls.builtins.code_actions.shellcheck)
end

if vim.fn.executable("deno") == 1 then
  nvim_lsp.denols.setup({
    on_attach = on_attach,
  });
  table.insert(null_ls_sources, null_ls.builtins.formatting.deno_fmt);
end

if vim.fn.executable("node_modules/.bin/tsc") == 1 then
  -- this provides auto-import on completion, among other things
  local ts_utils = require("nvim-lsp-ts-utils")

  nvim_lsp.tsserver.setup({
    init_options = ts_utils.init_options,
    flags = {
      debounce_text_changes = 150,
    },
    on_attach = function(client, bufnr)
      -- mark tsserver as not having formatting available as we rely on
      -- null-ls/eslint for that and having both available makes nvim ask us
      -- which LS to use everytime we format
      client.resolved_capabilities.document_formatting = false
      client.resolved_capabilities.document_range_formatting = false

      -- settings here are buffer-local so has to be run on_attach
      ts_utils.setup({
        debug = false,
        disable_commands = false,
        enable_import_on_completion = true,

        -- import all
        import_all_timeout = 5000, -- ms
        -- lower numbers = higher priority
        import_all_priorities = {
            same_file = 1, -- add to existing import statement
            local_files = 2, -- git files or files with relative path markers
            buffer_content = 3, -- loaded buffer content
            buffers = 4, -- loaded buffer names
        },
        import_all_scan_buffers = 100,
        import_all_select_source = false,

        -- filter diagnostics
        filter_out_diagnostics_by_severity = {},
        filter_out_diagnostics_by_code = {},

        -- inlay hints
        auto_inlay_hints = false,
        inlay_hints_highlight = "Comment",

        -- update imports on file move
        update_imports_on_move = false,
        require_confirmation_on_move = false,
        watch_dir = nil,
      })

      -- required to fix code action ranges and filter diagnostics
      ts_utils.setup_client(client)

      on_attach(client, bufnr)
    end,
  })
end

if vim.fn.executable("solargraph") == 1 then
  nvim_lsp.solargraph.setup({
    on_attach = on_attach,
  })
end

local group = vim.api.nvim_create_augroup('LspFormatting', { clear = false })

null_ls.setup({
  sources = null_ls_sources,

  on_attach = function(client, bufnr)
    -- format on save
    if client.resolved_capabilities.document_formatting then
      for key, cmd in pairs(vim.api.nvim_get_autocmds({ group = group, buffer = bufnr })) do
        vim.api.nvim_del_autocmd(cmd.id)
      end
      vim.api.nvim_create_autocmd('BufWritePre', {
        group = group,
        buffer = bufnr,
        callback = vim.lsp.buf.formatting_sync,
      })
    end
  end,
});

-- show LSP progress bar
require('fidget').setup()
